-- SLDataText Statline Copyright (c) 2011 Gall "Suicidal Katt" Levi <suicidalkatt@gmail.com> All rights reserved.
-- Original SLDataText Statline Copyright (c) 2008, 2009 Jeff "Taffu" Fancher <jdfancher@gmail.com> All rights reserved.
-- Version: 3.0 -- Rev Date: 8/17/2011
-- Changes in this version include:
--		1. Fixed issue with text being clamped and changing to '...' if out of width.
--		2. Removed text alignment option since it is not possible with this module.
--		3. Fixed all stat value outputs and including many additional new values.
--		4. Added spacing option between statblocks.
--		5. Removed some no longer used stats. (Armor Penetration, Defense Rating)

local SLDataText = LibStub("AceAddon-3.0"):GetAddon("SLDataText")
local L = LibStub("AceLocale-3.0"):GetLocale("SLDataText")
local media = LibStub("LibSharedMedia-3.0")
local justTable, strataTable, pointTable = SLDataText.just, SLDataText.strata, SLDataText.point
local db

local MODNAME = "Statline"
local Statline = SLDataText:NewModule(MODNAME, "AceEvent-3.0")

local curTalents = nil
--local GetActiveTalentGroup = GetActiveTalentGroup() 

local optGetter, optSetter
do
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
        SLDataText:RefreshModule(Statline)
	end
end

local statistics = {
-- General
	["Strength"] = L["Strength"],
	["Agility"] = L["Agility"],
	["Intellect"] = L["Intellect"],
	["Stamina"] = L["Stamina"],
	["Spirit"] = L["Spirit"],
	["Mana Regen"] = L["Mana Regen"],
	["Energy Regen"] = L["Energy Regen"],
	["Mastery Rating"] = L["Mastery Rating"],
	["Mastery Bonus"] = L["Mastery Bonus"],
	["Average Item Level"] = L["Average Item Level"],
	["Movement Speed"] = L["Movement Speed"],
-- Spell
	["Spell Damage"] = L["Spell Damage"],
	["Spell Healing"] = L["Spell Healing"],
	["Spell Crit Rating"] = L["Spell Crit Rating"],
	["Spell Crit Percent"] = L["Spell Crit Percent"],
	["Spell Haste Rating"] = L["Spell Haste Rating"],
	["Spell Haste Percent"] = L["Spell Haste Percent"],
	["Spell Hit Rating"] = L["Spell Hit Rating"],
	["Spell Hit Percent"] = L["Spell Hit Percent"],
	["Spell Penetration"] = L["Spell Penetration"],
	["Melee Attack Power"] = L["Melee Attack Power"],
	["Melee Attack Speed"] = L["Melee Attack Speed"],
	["Melee Crit Rating"] = L["Melee Crit Rating"],
	["Melee Crit Percent"] = L["Melee Crit Percent"],
	["Melee Haste Rating"] = L["Melee Haste Rating"],
	["Melee Haste Percent"] = L["Melee Haste Percent"],
	["Melee Hit Rating"] = L["Melee Hit Rating"],
	["Melee Hit Percent"] = L["Melee Hit Percent"],
	["Melee Expertise Rating"] = L["Melee Expertise Rating"],
	["Melee Expertise Per Weapon"] = L["Melee Expertise Per Weapon"],
	["Melee Expertise Percent"] = L["Melee Expertise Percent"],
-- Ranged
	["Ranged Attack Power"] = L["Ranged Attack Power"],
	["Ranged Attack Speed"] = L["Ranged Attack Speed"],
	["Ranged Crit Rating"] = L["Ranged Crit Rating"],
	["Ranged Crit Percent"] = L["Ranged Crit Percent"],
	["Ranged Haste Rating"] = L["Ranged Haste Rating"],
	["Ranged Haste Percent"] = L["Ranged Haste Percent"],
	["Ranged Hit Rating"] = L["Ranged Hit Rating"],
	["Ranged Hit Percent"] = L["Ranged Hit Percent"],
-- Defense	
	["Armor"] = L["Armor"],
	["Parry Rating"] = L["Parry Rating"],
	["Parry Percent"] = L["Parry Percent"],
	["Dodge Rating"] = L["Dodge Rating"],
	["Dodge Percent"] = L["Dodge Percent"],
	["Block Rating"] = L["Block Rating"],
	["Block Percent"] = L["Block Percent"],
	["Resilience Rating"] = L["Resilience Rating"],
	["Resilience Percent"] = L["Resilience Percent"],
}

-- [[General]]--

local function GetStrength()
    local color = SLDataText:GetColor()
    local stre = UnitStat("player", 1)
    return format("|cff%s%s|r %d", color, L["Str"], stre)
end

local function GetAgility()
    local color = SLDataText:GetColor()
    local agil = UnitStat("player", 2)
    return format("|cff%s%s|r %d", color, L["Agi"], agil)
end

local function GetStamina()
    local color = SLDataText:GetColor()
    local sta = UnitStat("player", 3)
    return format("|cff%s%s|r %d", color, L["Stam"], sta)
end

local function GetIntellect()
    local color = SLDataText:GetColor()
    local intel = UnitStat("player", 4)
    return format("|cff%s%s|r %d", color, L["Int"], intel)
end

local function GetSpirit()
    local color = SLDataText:GetColor()
    local spr = UnitStat("player", 5)
    return format("|cff%s%s|r %d", color, L["Spirit"], spr)
end

local function GetManaRegeneration()
    local color = SLDataText:GetColor()
    local base, cast = GetManaRegen()
    return format("|cff%s%s|r %d", color, L["MP5"], floor(cast*5))
end

local function GetEnergyRegeneration()
	local color = SLDataText:GetColor()
    local base = GetPowerRegen()
    return format("|cff%s%s|r %d", color, L["EnReg"], base)
end

local function GetMasteryRating()
    local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_MASTERY)
    return format("|cff%s%s|r %d", color, L["MastRat"], val)
end

local function GetMasteryBonus()
	local color = SLDataText:GetColor()
	local val = GetMastery()
	return format("|cff%s%s|r %.2f", color, L["MastBns"], val)
end

local function GetAverageILevel()
	local color = SLDataText:GetColor()
    local val = GetAverageItemLevel()
    return format("|cff%s%s|r %d", color, L["iLvl"], floor(val))
end

local f = CreateFrame("Frame")
f.time = 0
f.interval = 1
local MovementSpeed
local MS_ON
local function GetMovementSpeed()
	local color = SLDataText:GetColor()
	local _, runSpeed, flightSpeed, swimSpeed = GetUnitSpeed("player");
	local wasSwimming
	runSpeed = runSpeed/BASE_MOVEMENT_SPEED*100;
	flightSpeed = flightSpeed/BASE_MOVEMENT_SPEED*100;
	swimSpeed = swimSpeed/BASE_MOVEMENT_SPEED*100;
	-- Determine whether to display running, flying, or swimming speed
	local speed = runSpeed;
	local swimming = IsSwimming(unit);
	if (swimming) then
		speed = swimSpeed;
	elseif (IsFlying(unit)) then
		speed = flightSpeed;
	end
	--statFrame.Value:SetFormattedText("%d%%", speed+0.5);
	MovementSpeed = format("|cff%s%s|r %.1f%%", color, L["MovSpd"], speed)
	return format("|cff%s%s|r %.1f%%", color, L["MovSpd"], speed)
end

function Statline:EnableMoveSpeed()
	f:SetScript("OnUpdate", function(self,elapsed)
		f.time = f.time + elapsed
		if f.time > f.interval then
			GetMovementSpeed()
			Statline:Refresh()
			f.time = 0
		end
	end)
	if not MS_ON then 
		MS_ON = true
	end
end

function Statline:DisableMoveSpeed()
	f:SetScript("OnUpdate",nil)
	MS_ON = nil
end
-- [[Spell]]--

local function GetSpellDamage()
    -- 1 = Physical, 2 = Holy, 3 = Fire, 4 = Nature, 5 = Frost, 6 = Shadow, 7 = Arcane
    local color = SLDataText:GetColor()
    local minDmg = GetSpellBonusDamage(2)
    for i = 3, 7 do
        local splDmg = GetSpellBonusDamage(i)
        minDmg = min(minDmg, splDmg)
    end
    return format("|cff%s%s|r %d", color, L["SpDmg"], minDmg)
end

local function GetSpellHealing()
    local color = SLDataText:GetColor()
    local val = GetSpellBonusHealing()
    return format("|cff%s%s|r %d", color, L["SpHeal"], val)
end

local function GetSpellCritRating()
    local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_CRIT_SPELL)
    return format("|cff%s%s|r %.d", color, L["SpCrRat"], val)
end

local function GetSpellCritPercent()
    local color = SLDataText:GetColor()
	local start = 2
	local holySchool = 2;
	-- Start at 2 to skip physical damage
	local minCrit = GetSpellCritChance(start);
	local spellCrit
	local values = {}
	for i=(start+1), MAX_SPELL_SCHOOLS do
		spellCrit = GetSpellCritChance(i);
		minCrit = min(minCrit, spellCrit);
	end
	return format("|cff%s%s|r %.1f%%", color, L["SpCrPerc"], minCrit)
end

local function GetSpellHasteRating()
    local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_HASTE_SPELL);    
    return format("|cff%s%s|r %d", color, L["SpHastRat"], val)
end

local function GetSpellHastePercent()
	local color = SLDataText:GetColor()
    local val = UnitSpellHaste("player");    
    return format("|cff%s%s|r %.1f%%", color, L["SpHastPerc"], val)
end

local function GetSpellHitRating()
    local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_HIT_SPELL)
    return format("|cff%s%s|r %d", color, L["SpHitRat"], val)
end

local function GetSpellHitPercent()
    local color = SLDataText:GetColor()
    local val = GetCombatRatingBonus(CR_HIT_SPELL) + GetSpellHitModifier()
    return format("|cff%s%s|r %.1f%%", color, L["SpHitPerc"], val)
end

local function GetSpellPenetrationRating()
    local color = SLDataText:GetColor()
    local val = GetSpellPenetration() -- %
    return format("|cff%s%s|r %.1f%%", color, L["SpPen"], val)
end

-- [[Melee]] --

local function GetMeleeAttackPower()
    local color = SLDataText:GetColor()
    local base, pos, neg = UnitAttackPower("player")
    return format("|cff%s%s|r %d", color, L["MAttPwr"], ((base+pos)-neg))
end

local function GetMeleeAttackSpeed()
    local color = SLDataText:GetColor()
    local mainhand, offhand = UnitAttackSpeed("player")
    local speed
    if ( offhand and offhand > 0 ) then
        speed = format("%.2f/%.2f", mainhand, offhand)
    else
        speed = format("%.2f", mainhand)
    end
    return format("|cff%s%s|r %s", color, L["MAttSpd"], speed)
end
--
local function GetMeleeCritRating()
    local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_CRIT_MELEE)
    return format("|cff%s%s|r %d", color, L["MCrRat"], val)
end

local function GetMeleeCritPercent()
    local color = SLDataText:GetColor()
    local val = GetCritChance() -- %
    return format("|cff%s%s|r %.1f%%", color, L["MCrPerc"], val)
end
--
local function GetMeleeHasteRating()
	local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_HASTE_MELEE)
    return format("|cff%s%s|r %d", color, L["MHastRat"], val)
end

local function GetMeleeHastePercent()
    local color = SLDataText:GetColor()
    local val = GetCombatRatingBonus(CR_HASTE_MELEE)
    return format("|cff%s%s|r %.1f%%", color, L["MHastPerc"], val)
end
--
local function GetMeleeHitRating()
	local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_HIT_MELEE)
    return format("|cff%s%s|r %d", color, L["MHitRat"], val)
end

local function GetMeleeHitPercent()
    local color = SLDataText:GetColor()
    local val = GetCombatRatingBonus(CR_HIT_MELEE)
    return format("|cff%s%s|r %.1f%%", color, L["MHitPerc"], val)
end

local function GetMeleeExpertiseRating()
	local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_EXPERTISE)
	return format("|cff%s%s|r %d", color, L["MExpRat"], val)
end

local function GetMeleeExpertisePerWeapon()
	local color = SLDataText:GetColor()
	local mainhand, offhand = GetExpertise();
	if ( IsDualWielding() ) then
        expertPerWeap = format("%.0f/%.0f", mainhand, offhand)
    else
        expertPerWeap = format("%.1f", mainhand)
    end
    return format("|cff%s%s|r %s", color, L["MExpPWeap"], expertPerWeap)
end

local function GetMeleeExpertisePercent()
	local color = SLDataText:GetColor()
	local mainhand, offhand = GetExpertisePercent();	
	if ( IsDualWielding() ) then
        expertPercent = format("%.1f%%(%.1f%%)", mainhand, offhand)
    else
        expertPercent = format("%.1f%%", mainhand)
    end
    return format("|cff%s%s|r %s", color, L["MExpPerc"], expertPercent)
end

-- [[Ranged]]--

local function GetRangedAttackPower()
    local color = SLDataText:GetColor()
    local base, pos, neg = UnitRangedAttackPower("player")
    return format("|cff%s%s|r %d", color, L["RAttPwr"], ((base+pos+neg)))
end

local function GetRangedAttackSpeed()
	local color = SLDataText:GetColor()
    local val = UnitRangedDamage("player")
	return format("|cff%s%s|r %.2f", color, L["RAttSpd"], val)
end

local function GetRangedCritRating()
    local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_CRIT_RANGED)
    return format("|cff%s%s|r %d", color, L["RCrRat"], val)
end

local function GetRangedCritPercent()
    local color = SLDataText:GetColor()
    local val = GetRangedCritChance()
    return format("|cff%s%s|r %.1f%%", color, L["RCrPerc"], val)
end

local function GetRangedHasteRating()
    local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_HASTE_RANGED)
    return format("|cff%s%s|r %d", color, L["RHastRat"], val)
end

local function GetRangedHastePercent()
    local color = SLDataText:GetColor()
    local val = GetCombatRatingBonus(CR_HASTE_RANGED)
    return format("|cff%s%s|r %.1f%%", color, L["RHastPerc"], val)
end

local function GetRangedHitRating() 
	local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_HIT_RANGED)
    return format("|cff%s%s|r %d", color, L["RHitRat"], val)
end

local function GetRangedHitPercent() 
	local color = SLDataText:GetColor()
    local val = GetCombatRatingBonus(CR_HIT_RANGED)
    return format("|cff%s%s|r %.1f%%", color, L["RHitPerc"], val)
end
-- [[Defense]]--
local function GetArmorValue()
    local color = SLDataText:GetColor()
    local base, effective, val, _, _ = UnitArmor("player")
    return format("|cff%s%s|r %d", color, L["Arm"], val)
end
--
local function GetParryRating() 
	local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_PARRY)
    return format("|cff%s%s|r %d", color, L["ParRat"], val)
end

local function GetParryPercent() 
	local color = SLDataText:GetColor()
    local val = GetCombatRatingBonus(CR_PARRY)
    return format("|cff%s%s|r %.1f%%", color, L["ParPerc"], val)
end
--
local function GetDodgeRating()
	local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_DODGE)
    return format("|cff%s%s|r %d", color, L["DodgRat"], val)
end

local function GetDodgePercent() 
	local color = SLDataText:GetColor()
    local val = GetCombatRatingBonus(CR_DODGE)
    return format("|cff%s%s|r %.1f%%", color, L["DodgPerc"], val)
end
--
local function GetBlockRating()
	local color = SLDataText:GetColor()
    local val = GetCombatRating(CR_BLOCK)
    return format("|cff%s%s|r %d", color, L["BlkRat"], val)
end

local function GetBlockPercent() 
	local color = SLDataText:GetColor()
    local val = GetCombatRatingBonus(CR_BLOCK)
    return format("|cff%s%s|r %.1f%%", color, L["BlkPerc"], val)
end
--
local function GetResilienceRating()
    local color = SLDataText:GetColor()
    local val = GetCombatRating(COMBAT_RATING_RESILIENCE_PLAYER_DAMAGE_TAKEN)
	return format("|cff%s%s|r %d", color, L["ResRat"], val)
end

local function GetResiliencePercent()
	local color = SLDataText:GetColor()
    local val = GetCombatRatingBonus(COMBAT_RATING_RESILIENCE_PLAYER_DAMAGE_TAKEN)
    return format("|cff%s%s|r %.1f%%", color, L["ResPerc"], val)
end

local options
local function getOptions()
	if not options then options = {
        type = "group",
        name = L["Statline"],
        arg = MODNAME,
        get = optGetter,
        set = optSetter,
        args = {
            genHeader = {
                type = "header",
                name = L["GenSLSet"],
                order = 25,
            },
            enabled = {
                type = "toggle",
                name = L["Enabled"],
                desc = L["EnabledDesc"],
                get = function() return SLDataText:GetModuleEnabled(MODNAME) end,
                set = function(info, value) 
                    SLDataText:SetModuleEnabled(MODNAME, value)
                    if ( SLDataText:GetModuleEnabled(MODNAME) ) then
                        Statline:PLAYER_ENTERING_WORLD()
                    end
                end,
                order = 50,
            },
            hideTooltip = {
                type = "toggle",
                name = L["HideTT"],
                desc = L["HideTTDesc"],
                order = 100,
            },
            noCombatHide = {
                type = "toggle",
                name = L["SIC"],
                desc = L["SICDesc"],
                order = 150,
            },
            dispHeader = {
                type = "header",
                name = L["DispSet"],
                order = 200,
            },
            stat1 = {
                type = "select",
                name = L["Stat1"],
                desc = L["Stat1Desc"],
                values = statistics,
                get = function()
                    local stat1
                    if ( GetActiveTalentGroup() == 1 ) then
                        stat1 = db.primary.stat1
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        stat1 = db.secondary.stat1
                    end
                    return stat1
                end,
                set = function(_, value)
                    if ( GetActiveTalentGroup() == 1 ) then
                        db.primary.stat1 = value
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        db.secondary.stat1 = value
                    end
                    Statline:Refresh()
                end,
                width = "double",
                order = 220,
            },
            stat2 = {
                type = "select",
                name = L["Stat2"],
                desc = L["Stat2Desc"],
                values = statistics,
                get = function()
                    local stat1
                    if ( GetActiveTalentGroup() == 1 ) then
                        stat1 = db.primary.stat2
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        stat1 = db.secondary.stat2
                    end
                    return stat1
                end,
                set = function(_, value)
                    if ( GetActiveTalentGroup() == 1 ) then
                        db.primary.stat2 = value
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        db.secondary.stat2 = value
                    end
                    Statline:Refresh()
                end,
                width = "double",
                order = 240,
            },
            stat3 = {
                type = "select",
                name = L["Stat3"],
                desc = L["Stat3Desc"],
                values = statistics,
                get = function()
                    local stat1
                    if ( GetActiveTalentGroup() == 1 ) then
                        stat1 = db.primary.stat3
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        stat1 = db.secondary.stat3
                    end
                    return stat1
                end,
                set = function(_, value)
                    if ( GetActiveTalentGroup() == 1 ) then
                        db.primary.stat3 = value
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        db.secondary.stat3 = value
                    end
                    Statline:Refresh()
                end,
                width = "double",
                order = 260,
            },
            useGlobalFont = {
                type = "toggle",
                name = L["UseGblFont"],
                desc = L["UseGblFontDesc"],
                order = 300,
            },
            useGlobalFontSize = {
                type = "toggle",
                name = L["UseGblFSize"],
                desc = L["UseGblFSizeDesc"],
                order = 350,
            },
            fontFace = {
                type = "select",
                name = L["Font"],
                desc = L["FontDesc"],
                disabled = function()
                    local isTrue
                    if ( db.useGlobalFont ) then isTrue = true else isTrue = false end
                    return isTrue
                end,
                values = media:List("font"),
                get = function()
                    for k, v in pairs(media:List("font")) do
                        if db.fontFace == v then
                            return k
                        end
                    end
                end,
                set = function(_, font)
                    local list = media:List("font")
                    db.fontFace = list[font]
                    SLDataText:RefreshModule(Statline)
                end,
                width = "double",
                order = 600,
            },
            fontSize = {
                type = "range",
                name = L["FontSize"],
                desc = L["FontSizeDesc"],
                disabled = function()
                    local isTrue
                    if ( db.useGlobalFontSize ) then isTrue = true else isTrue = false end
                    return isTrue
                end,
                min = 6, max = 36, step = 1,
                width = "double",
                order = 650,
            },
            posHeader = {
                type = "header",
                name = L["LaySet"],
                order = 700,
            },
            spacing = {
				type = "range",
				name = "Text Spacing",
				desc = "Set the spacing between stat text blocks",
				width = "double",
				get = function() return db.spacing end,
				min = 0,
				max = 250,
				step = 1,
				order = 750,
			},
            anchor = {
                type = "input",
                name = L["ParFrm"],
                desc = L["ParFrmDesc"],
                get = function() return db.anchor end,
                width = "double",
                order = 800,
            },
            anchorFrom = {
                type = "select",
                name = L["AnchFrom"],
                desc = L["AnchFromDesc"],
                values = pointTable,
                get = function() return db.anchorFrom end,
                width = "double",
                order = 900,
            },
            offX = {
                type = "input",
                name = L["XOff"],
                desc = L["XOffDesc"],
                get = function() return tostring(db.offX) end,
                width = "double",
                order = 1000,
            },
            offY = {
                type = "input",
                name = L["YOff"],
                desc = L["YOffDesc"],
                get = function() return tostring(db.offY) end,
                width = "double",
                order = 1100,
            },
            strata = {
                type = "select",
                name = L["Strata"],
                desc = L["StrataDesc"],
                values = strataTable,
                width = "double",
                order = 1600,
            },
        },
    }
	end
	
	return options
end

local function buildModule(self)
    if ( not self.frame ) then self.frame = CreateFrame("Frame", "SLDT_Statline", UIParent) end -- The frame
    -- Blocks
    if ( not self.block1 ) then self.block1 = CreateFrame("Frame", nil, self.frame) end
    if ( not self.string1 ) then self.string1 = self.block1:CreateFontString(nil, "OVERLAY"); self.string1:SetPoint("LEFT",self.block1,"LEFT"); self.string1:SetJustifyH("LEFT") end
    if ( not self.block2 ) then self.block2 = CreateFrame("Frame", nil, self.frame) end
    if ( not self.string2 ) then self.string2 = self.block2:CreateFontString(nil, "OVERLAY"); self.string2:SetPoint("CENTER",self.block2,"CENTER"); self.string1:SetJustifyH("CENTER") end
    if ( not self.block3 ) then self.block3 = CreateFrame("Frame", nil, self.frame) end
    if ( not self.string3 ) then self.string3 = self.block3:CreateFontString(nil, "OVERLAY"); self.string3:SetPoint("RIGHT",self.block3,"RIGHT"); self.string1:SetJustifyH("RIGHT") end
    
    if ( not self.string ) then self.string = self.frame:CreateFontString(nil, "OVERLAY") end -- The font string (dummy)
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function Statline:PLAYER_ENTERING_WORLD()
    local enabled = SLDataText:GetModuleEnabled(MODNAME)
    if ( enabled ) then
        
    end
    SLDataText:RefreshModule(self)
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function Statline:OnInitialize()
	self.db = SLDataText.db:RegisterNamespace(MODNAME)
    self.db:RegisterDefaults({
        profile = {
            primary = {
                stat1 = "Spell Healing",
                stat2 = "Mana Regen",
                stat3 = "Spell Crit Percent",
            },
            secondary = {
                stat1 = "Armor",
                stat2 = "Block Percent",
                stat3 = "Dodge Percent",
            },
            hideTooltip = false,
            noCombatHide = false,
            useGlobalFont = true,
            fontFace = "Arial Narrow",
            useGlobalFontSize = true,
            fontSize = 12,
            justify = "CENTER",
			spacing = 10,
            anchorPoint = "CENTER", 
            anchor = "UIParent", 
            anchorFrom = "CENTER", 
            offX = 0, 
            offY = 0,
            strata = "BACKGROUND",
        },
    })
	db = self.db.profile
    
    if ( not self.isMoving ) then self.isMoving = false end
	self:SetEnabledState(SLDataText:GetModuleEnabled(MODNAME))
	SLDataText:RegisterModuleOptions(MODNAME, getOptions)
end

-- Stat functions ---

-- Stat/Func Tbl ----

local statFunc 
local function RefreshStats()
	-- Update periodic stats
	GetMovementSpeed()
	
    if ( statFunc ) then statFunc = nil end
    statFunc = {
	-- General
        ["Strength"] = function() return GetStrength() end,
		["Agility"] = function() return GetAgility() end,
		["Intellect"] = function() return GetIntellect() end,
		["Stamina"] = function() return GetStamina() end,
		["Spirit"] = function() return GetSpirit() end,
		["Mana Regen"] = function() return GetManaRegeneration() end,
		["Energy Regen"] = function() return GetEnergyRegeneration() end,
		["Mastery Rating"] = function() return GetMasteryRating() end,
		["Mastery Bonus"] = function() return GetMasteryBonus() end,
		["Average Item Level"] = function() return GetAverageILevel() end,
		["Movement Speed"] = function() return MovementSpeed end,
	-- Spell
		["Spell Damage"] = function() return GetSpellDamage() end,
		["Spell Healing"] = function() return GetSpellHealing() end,
		["Spell Crit Rating"] = function() return GetSpellCritRating() end,
		["Spell Crit Percent"] = function() return GetSpellCritPercent() end,
		["Spell Haste Rating"] = function() return GetSpellHasteRating() end,
		["Spell Haste Percent"] = function() return GetSpellHastePercent() end,
		["Spell Hit Rating"] = function() return GetSpellHitRating() end,
		["Spell Hit Percent"] = function() return GetSpellHitPercent() end,
		["Spell Penetration"] = function() return GetSpellPenetrationRating() end,
		["Melee Attack Power"] = function() return GetMeleeAttackPower() end,
		["Melee Attack Speed"] = function() return GetMeleeAttackSpeed() end,
		["Melee Crit Rating"] = function() return GetMeleeCritRating() end,
		["Melee Crit Percent"] = function() return GetMeleeCritPercent() end,
		["Melee Haste Rating"] = function() return GetMeleeHasteRating() end,
		["Melee Haste Percent"] = function() return GetMeleeHastePercent() end,
		["Melee Hit Rating"] = function() return GetMeleeHitRating() end,
		["Melee Hit Percent"] = function() return GetMeleeHitPercent() end,
		["Melee Expertise Rating"] = function() return GetMeleeExpertiseRating() end,
		["Melee Expertise Per Weapon"] = function() return GetMeleeExpertisePerWeapon() end,
		["Melee Expertise Percent"] = function() return GetMeleeExpertisePercent() end,
	-- Ranged
		["Ranged Attack Power"] = function() return GetRangedAttackPower() end,
		["Ranged Attack Speed"] = function() return GetRangedAttackSpeed() end,
		["Ranged Crit Rating"] = function() return GetRangedCritRating() end,
		["Ranged Crit Percent"] = function() return GetRangedCritPercent() end,
		["Ranged Haste Rating"] = function() return GetRangedHasteRating() end,
		["Ranged Haste Percent"] = function() return GetRangedHastePercent() end,
		["Ranged Hit Rating"] = function() return GetRangedHitRating() end,
		["Ranged Hit Percent"] = function() return GetRangedHitPercent() end,
	-- Defense	
		["Armor"] = function() return GetArmorValue() end,
		["Parry Rating"] = function() return GetParryRating() end,
		["Parry Percent"] = function() return GetParryPercent() end,
		["Dodge Rating"] = function() return GetDodgeRating() end,
		["Dodge Percent"] = function() return GetDodgePercent() end,
		["Block Rating"] = function() return GetBlockRating() end,
		["Block Percent"] = function() return GetBlockPercent() end,
		["Resilience Rating"] = function() return GetResilienceRating() end,
		["Resilience Percent"] = function() return GetResiliencePercent() end,
    }
end

-- Stat Fetch Func --

local function GatherStats(stat1, stat2, stat3)
    if statFunc == nil then 
		RefreshStats()
	end
    return statFunc[stat1](), statFunc[stat2](), statFunc[stat3]()
end

function Statline:Update_Talents()
	curTalents = GetActiveTalentGroup()
end

---------------------

function Statline:OnEnable()
    buildModule(self)
    --self:RegisterEvent("UNIT_AURA", "Refresh")
    self:RegisterEvent("UNIT_STATS", "Refresh")
    self:RegisterEvent("UNIT_DAMAGE", "Refresh")
    self:RegisterEvent("UNIT_RANGEDDAMAGE", "Refresh")
    self:RegisterEvent("PLAYER_DAMAGE_DONE_MODS", "Refresh")
    self:RegisterEvent("UNIT_ATTACK_SPEED", "Refresh")
    self:RegisterEvent("UNIT_ATTACK_POWER", "Refresh")
    self:RegisterEvent("UNIT_RANGED_ATTACK_POWER", "Refresh")
    self:RegisterEvent("UNIT_ATTACK", "Refresh")
    self:RegisterEvent("UNIT_RESISTANCES", "Refresh")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED","Update_Talents")
    if ( not self.frame:IsShown() ) then self.frame:Show() end
end

function Statline:OnDisable()
    self:UnregisterEvent("UNIT_STATS")
    self:UnregisterEvent("UNIT_DAMAGE")
    self:UnregisterEvent("UNIT_RANGEDDAMAGE")
    self:UnregisterEvent("PLAYER_DAMAGE_DONE_MODS")
    self:UnregisterEvent("UNIT_ATTACK_SPEED")
    self:UnregisterEvent("UNIT_ATTACK_POWER")
    self:UnregisterEvent("UNIT_RANGED_ATTACK_POWER")
    self:UnregisterEvent("UNIT_ATTACK")
    self:UnregisterEvent("UNIT_RESISTANCES")
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	if ( self.frame:IsShown() ) then self.frame:Hide() end
end

function Statline:Refresh()
    local stat1, stat2, stat3 --= "Stat 1","Stat 2","Stat 3"
	if not curTalents then self:Update_Talents() end
	if ( curTalents == 1 ) then
        stat1, stat2, stat3 = GatherStats(db.primary.stat1, db.primary.stat2, db.primary.stat3)
		if db.primary.stat1 == "Movement Speed" or db.primary.stat2 == "Movement Speed" or db.primary.stat3 == "Movement Speed" then
			if not MS_ON then
				self:EnableMoveSpeed()
			end
		else
			if MS_ON then
				self:DisableMoveSpeed()
			end
		end
    elseif ( curTalents == 2 ) then
        stat1, stat2, stat3 = GatherStats(db.secondary.stat1, db.secondary.stat2, db.secondary.stat3)
		if db.secondary.stat1 == "Movement Speed" or db.secondary.stat2 == "Movement Speed" or db.secondary.stat3 == "Movement Speed" then
			if not MS_ON then
				self:EnableMoveSpeed()
			end
		else
			if MS_ON then
				self:DisableMoveSpeed()
			end
		end
    end
    
    if ( SLDataText.db.profile.locked and not db.hideTooltip ) then
        self.frame:EnableMouse(true)
        self.frame:SetScript("OnEnter", function(self)
            GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
            GameTooltip:AddLine("|cffffff00Statline|r")
			GameTooltip:AddLine("-------------------------", 1, 1, 1)
			local first
			for k,v in pairs(statFunc) do
				if not first then 
					first = statFunc[k]() 
				else
					GameTooltip:AddDoubleLine(first,statFunc[k](), 1, 1, 1, 1, 1, 1)
					first = nil
				end
			end
            GameTooltip:Show()
        end)
        self.frame:SetScript("OnLeave", function()
            if ( GameTooltip:IsShown() ) then
                GameTooltip:Hide()
            end
        end)
    else
        self.frame:EnableMouse(false)
        self.frame:SetScript("OnEnter", nil)
        self.frame:SetScript("OnLeave", nil)
    end
    
    -- Update main blocks and strings
    local font, fontH = self.string:GetFont()
    -- Stat 1
    self.string1:SetFont(font, fontH)
    self.string1:SetShadowColor(0, 0, 0, 1)
    self.string1:SetShadowOffset(1.5, -1.5)
    
    self.string1:SetText(stat1)
    self.block1:SetWidth(self.string1:GetStringWidth())
    self.block1:SetHeight(self.string1:GetHeight())
    --self.string1:SetAllPoints(self.block1)
    -- Stat 2
    self.string2:SetFont(font, fontH)
    self.string2:SetShadowColor(0, 0, 0, 1)
    self.string2:SetShadowOffset(1.5, -1.5)
    
    self.string2:SetText(stat2)
    self.block2:SetWidth(self.string2:GetStringWidth())
    self.block2:SetHeight(self.string2:GetHeight())
    --self.string2:SetAllPoints(self.block2)
    -- Stat 3
    self.string3:SetFont(font, fontH)
    self.string3:SetShadowColor(0, 0, 0, 1)
    self.string3:SetShadowOffset(1.5, -1.5)
    
    self.string3:SetText(stat3)
    self.block3:SetWidth(self.string3:GetStringWidth())
    self.block3:SetHeight(self.string3:GetHeight())
    --self.string3:SetAllPoints(self.block3)
    
    -- Align blocks
    self.block2:ClearAllPoints()
    self.block2:SetPoint("CENTER", self.frame, "CENTER", 0, 0)
	self.block1:ClearAllPoints()
    self.block1:SetPoint("RIGHT", self.block2, "LEFT", -(db.spacing), 0)
    
    self.block3:ClearAllPoints()
    self.block3:SetPoint("LEFT", self.block2, "RIGHT", (db.spacing), 0)
    
    -- And then update the module for refreshing/resizing text/frame
    local color = SLDataText.db.profile.gColor
    local width = (self.string1:GetStringWidth()+self.string2:GetStringWidth()+self.string3:GetStringWidth()+4)
    self.string:SetWidth(width)
    self.string:SetHeight(self.string1:GetHeight())
    
    SLDataText:UpdateModule(self)
end